<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 - 2019 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 - 2019 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/
class MageBinary_BinaryPay_Block_Latitudepay_Snippet_Abstract extends Mage_Catalog_Block_Product_Abstract
{
    CONST LATITUDE_ENABLED   = "payment/latitudepay/active";
    CONST GENOAPAY_ENABLED   = "payment/genoapay/active";
    CONST LATITUDE_PAYMENT_SERVICES  = "payment/latitudepay/payment_services";
    CONST LATITUDE_PAYMENT_TERMS  = "payment/latitudepay/payment_terms";
    CONST LATITUDE_IMAGE_API_URL = "payment/latitudepay/images_api_url";
    CONST LATITUDE_CURRENCY  = "payment/latitudepay/currency";
    CONST GENOAPAY_CURRENCY  = "payment/genoapay/currency";

    protected $currentCurrencyCode;

    protected $latitudepayCurrency;

    protected $genoapayCurrency;

    protected function _construct()
    {
        parent::_construct();
        /** @noinspection PhpUnhandledExceptionInspection */
        $this->currentCurrencyCode          =  Mage::app()->getStore()->getCurrentCurrencyCode();
        $this->latitudepayCurrency          =  Mage::getStoreConfig(self::LATITUDE_CURRENCY);
        $this->genoapayCurrency             =  Mage::getStoreConfig(self::GENOAPAY_CURRENCY);
        $this->setTemplate('magebinary/binarypay/latitudepay/catalog/payments.phtml');
    }

    /**
     * Checks whether the Latitudepay payment method is enabled.
     *
     * @param null $store
     * @return mixed
     */
    public function isLatitudepayEnabled($store = null)
    {
        return Mage::getStoreConfig(self::LATITUDE_ENABLED, $store) && ($this->currentCurrencyCode == $this->latitudepayCurrency) ;
    }
    /**
     * get Genoapay Enabled
     *
     * @param null $store
     * @return mixed
     */
    public function isGenoapayEnabled($store = null)
    {
        return Mage::getStoreConfig(self::GENOAPAY_ENABLED, $store) && ($this->currentCurrencyCode == $this->genoapayCurrency);
    }

    /**
     * Get Latitudepay Payment Services
     *
     * @param null $store
     * @return mixed
     */
    public function getLatitudepayPaymentServices($store = null)
    {
        if($this->isGenoapayEnabled()){
            return 'GPAY';
        }

        if($this->isLatitudepayEnabled()){
            if(Mage::getStoreConfig(self::LATITUDE_PAYMENT_SERVICES, $store)){
                return Mage::getStoreConfig(self::LATITUDE_PAYMENT_SERVICES, $store);
            }
            return 'LPAY';
        }
        return '';
    }

    /**
     * Get Latitudepay Payment Terms
     *
     * @param null $store
     * @return mixed
     */
    public function getLatitudepayPaymentTerms($store = null)
    {
        if($this->isLatitudepayEnabled()){
            return Mage::getStoreConfig(self::LATITUDE_PAYMENT_TERMS, $store);
        }
        return null;
    }

    /**
     * Get Image API URL.
     *
     * @param null $store
     * @return mixed
     */
    public function getImageApiUrl($store = null)
    {
        return Mage::getStoreConfig(self::LATITUDE_IMAGE_API_URL,$store);
    }

    /**
     * Retrieve util js
     * @throws \Magento\Framework\Exception\LocalizedException
     * @return \Magento\Framework\Phrase
     */
    public function getUtilJs()
    {
        /** @noinspection PhpUndefinedMethodInspection */
        return $this->getImageApiUrl().'/util.js';
    }

    /**
     * Gets Installment amount for current product
     * @return string
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getAmount()
    {
        $totalAmount = $this->getProduct()->getFinalPrice();
        return $totalAmount;
    }

    /**
     * Retrieve Snippet Image
     * @throws \Magento\Framework\Exception\LocalizedException
     * @return \Magento\Framework\Phrase
     */
    public function getSnippetImage()
    {
        /** @noinspection PhpUndefinedMethodInspection */
        $param = [
            'amount' => $this->getAmount(),
            'services' => $this->getLatitudepayPaymentServices(),
            'terms' => $this->getLatitudepayPaymentTerms(),
            'style' => 'default'
        ];
        return $this->getSnippetImageUrl() . '?' . http_build_query($param);
    }

    /**
     * Retrieve snippet image url
     * @throws \Magento\Framework\Exception\LocalizedException
     * @return \Magento\Framework\Phrase
     */
    public function getSnippetImageUrl()
    {
        /** @noinspection PhpUndefinedMethodInspection */
        return $this->getImageApiUrl().'/snippet.svg';
    }
}