<?php
class MageBinary_BinaryPay_Block_Sales_Order_Totals extends Mage_Core_Block_Template
{
    const BINARY_PAY_SURCHARGE_CODE = 'binarypay_surcharge';
    /**
     * $_surcharge: surcharge amount
     * @var float
     */
    protected $_surcharge;

    /**
     * Add Surcharge order total
     *
     * @return $this MageBinary_BinaryPay_Block_Sales_Order_Totals
     */
    public function initTotals()
    {
        if ($this->_hasSurcharge() && $this->_canApply()) {
            $this->_initSurcharge();
        }
        return $this;
    }

    protected function _canApply()
    {
        $payment = $this->getRequest()->getPost('payment');
        if (!isset($payment['method'])) {
            return false;
        }
        return in_array($payment['method'], MageBinary_BinaryPay_Model_Quote_Address_Total_Surcharge::BINARY_PAY_GATEWAY);
    }

    protected function _hasSurcharge()
    {
        $parent = $this->getParentBlock();
        $source  = $parent->getSource();

        $grandTotal     = (float) $source->getGrandTotal();
        $taxTotal       = (float) $source->getTaxAmount();
        $shippingTotal  = (float) $source->getShippingAmount();
        $subTotal       = (float) $source->getSubtotal();

        $surcharge  = $grandTotal - $subTotal - $taxTotal - $shippingTotal;

        if ($surcharge !== 0) {
            $this->setSurchargeAmount($surcharge);
            return true;
        }

        return false;
    }

    protected function _initSurcharge()
    {
        $surchargeTotal = new Varien_Object(
            array(
                'code'      => self::BINARY_PAY_SURCHARGE_CODE,
                'value'     => $this->getSurchargeAmount(),
                'label'     => $this->__('Transaction Fee')
            )
        );
        return $this->getParentBlock()->addTotal($surchargeTotal);
    }
}