<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/

/**
 * Abstract BinaryPay Controller
 */
abstract class MageBinary_BinaryPay_Controller_Abstract extends Mage_Core_Controller_Front_Action
{
    /**
     * $_gateway - Gateway Name for each controller
     */
    protected $_gateway;

    /**
     * $_data - save card info after build the card or mobile number details
     * @var array
     */
    protected $_data;

    /**
     * $_api - API model for current gateway
     */
    protected $_api;

    /**
     * $_tokenRef to delete
     * @var string
     */
    protected $_tokenRef;

     /**
     * @var Mage_Checkout_Model_Session
     */
    protected $_checkoutSession;

    /**
     * @var bool
     */
    protected $_debug = true;

    /**
     * @var array
     */
    protected $_response;

    /**
     * $_successStatuses
     * @var array
     */
    protected $_successStatuses = [];

    /**
     * $_pendingStatuses
     * @var array
     */
    protected $_pendingStatuses = [];

    /**
     * $_redirectUrl
     * @var string
     */
    protected $_redirectUrl;

    /**
     * @var Mage_Sales_Model_Quote
     */
    protected $_quote;

    /**
     * @var array
     */
    protected $_apiResponse;

    /**
     * Predispatch: should set layout area
     *
     * @return Mage_Core_Controller_Front_Action
     */
    public function preDispatch()
    {
        parent::preDispatch();

        if (!Mage::getSingleton('customer/session')->isLoggedIn() ||
            !Mage::getModel('magebinary_binarypay/directpost')->getPaymentMethodConfig('active') &&
            !Mage::getModel('magebinary_binarypay/webpayment')->getPaymentMethodConfig('active')) {
            $this->_redirect('/');
            return;
        }
    }

    /**
     * _getApi - Set API value and return Api model object
     * @return object
     */
    protected function _getApi()
    {
        if (!isset($this->_api) && !is_object($this->_api)) {
            $this->_api = Mage::getModel('magebinary_binarypay/'.$this->_gateway);
        }
        return $this->_api;
    }

    public function getApi()
    {
        return $this->_api = Mage::getModel('magebinary_binarypay/'.$this->_gateway);
    }

    /**
     * isPaid
     * @param  string  $status
     * @return boolean
     */
    public function isPaid($status)
    {
        return in_array($status, $this->_successStatuses);
    }

    /**
     * isPending
     * @param  string  $status
     * @return boolean
     */
    public function isPending($status)
    {
        return in_array($status, $this->_pendingStatuses);
    }

    public function addErrorException($session, $e)
    {
        $session->addError($e->getMessage());
        $session->setIsPending(true);
        $session->setPreventEmail(true);
        Mage::log($e->getTrace(), null, 'BinaryPay.log', true);
    }

    /**
     * _redirectAfterSuccess - Redirect user after successful operation
     */
    protected function _redirectAfterSuccess()
    {
        if ($redirectUrl = Mage::getSingleton('core/session')->getRedirectUrl(true)) {
            $this->_redirectUrl($redirectUrl);
        }
        return $this;
    }

    /**
     * _validateDataInfo - validate data info that user want to be saved
     * @return
     */
    protected function _validateDataInfo()
    {
        $data = $this->_data;
        // All fields should not be empty
        foreach ($data as $key => $value) {
            if (empty($value)) {
                Mage::throwException("Invalid key $key, value can not be empty");
                break;
            }
        }
        return $this;
    }

    /**
     * _prepareGetGateway - validate requested data ( Get and Post )
     * @return MageBinary_BinaryPay_Model_$this->_gateway
     */
    protected function _prepareGetGateway()
    {
        $request = $this->getRequest()->getPost();

        // If the request is not post
        if (empty($request)) {
            // try to get Data
            $request = $this->getRequest()->getParams();
            if (empty($request)) {
                Mage::throwException('Unknown requested data!');
            }
        }

        return $this;
    }

    /**
     * _validateDeleteRequest - validate delete request and set request reference value
     */
    protected function _validateDeleteRequest()
    {
        $request        = $this->getRequest();
        $params         = $request->getParams();

        if (empty($params['token_reference']) && empty($params['mobile_number'])) {
            Mage::throwException($this->__('Unknown Request!'));
        }

        if (!empty($params['token_reference'])) {
            $this->_tokenRef = $params['token_reference'];
        }

        return $this;
    }

    /**
     * Return checkout session object
     *
     * @return Mage_Checkout_Model_Session
     */
    protected function _getCheckoutSession()
    {
        if (!$this->_checkoutSession) {
            $this->_checkoutSession = Mage::getSingleton('checkout/session');
        }
        return $this->_checkoutSession;
    }

    /**
     * _sendMessage - send message to user in frontend
     * @param  array $message
     * @return $this
     */
    protected function _sendMessage($message)
    {
        if (isset($message) && !empty($message)) {
            $this->getResponse()->clearHeaders()->setHeader('Content-type', 'application/json', true);
            $this->getResponse()->setBody(json_encode($message));
        }

        return $this;
    }

    protected function _setGateway($gateway)
    {
        $this->_gateway = $gateway;
        return $this;
    }

    protected function isBackendOrder()
    {
        $isBackend = false;
        $cookie     = Mage::getModel('core/cookie');
        $session = $this->_getCheckoutSession();
        // If comming from backend
        $quoteId = $cookie->get('quoteEntityId');
        $cookie->delete('quoteEntityId');

        if ($quoteId) {
            $isBackend = true;
        }
        return $isBackend;
    }

    public function clearShoppingCart()
    {
        return Mage::getSingleton('checkout/cart')->truncate()->save();
    }

    public function getOnepage()
    {
        //check if firecheckout is used.
        $helper = Mage::helper('core');
        $checkModel = 'checkout/type_onepage';
        if ($helper->isModuleEnabled('TM_FireCheckout') && $helper->isModuleOutputEnabled('TM_FireCheckout')) {
            $checkModel = 'firecheckout/type_standard';
        };

        return Mage::getSingleton($checkModel);
    }

    public function placeOrder($exception = null)
    {
        try {
            $this->getOnepage()->setQuote($this->_getQuote())->initCheckout();
            $this->getOnepage()->getQuote()->collectTotals();
            $this->getOnepage()->saveOrder();
            $this->clearShoppingCart();

            if ($this->_getCheckoutSession()->getIsPending() && $exception) {
                $message = $exception->getMessage();
                if($exception->getMessage() == 'Your cart was updated resulting in a price mismatch. We have marked your order as Pending for a review.'){
                    $message = 'Amount paid and Total cart amount Mismatch. Please investigate before shipping.';
                }
                $this->_setOrderStatus(Mage_Sales_Model_Order::STATE_PENDING_PAYMENT, $message);
            }

            $this->_redirectUrl = MageBinary_BinaryPay_Model_Config::SUCCESSFUL_PAGE_URL;
        } catch (Mage_Core_Exception $e) {
            // Notice website administractor about the failed payment
            $this->processCannotCreateOrderError($e);
        } catch (Exception $e) {
            $this->processCannotCreateOrderError($e);
        }
        return $this;
    }

    public function processCannotCreateOrderError($e)
    {
        $session = Mage::getSingleton('checkout/session');
        Mage::helper('magebinary_binarypay')->sendCreateOrderFailedEmail(
            $this->_getQuote(),
            $e->getMessage()
        );
        $session->addError('Our system has failed to receive your order. Please contact us if your credit card has been charged.');
        $this->_redirectUrl = 'checkout/cart';
        return $this;
    }

    /**
     * setOrderStatus
     * Modify current order status
     * @param string $status
     */
    protected function _setOrderStatus($status, $message)
    {
        $order = Mage::getModel('sales/order')->loadByIncrementId($this->_getQuote()->getReservedOrderId());
        $order->setState($status, true);
        $order->addStatusHistoryComment($message);
        $order->save();
        return $this;
    }

    /**
     * Return checkout quote object
     *
     * @return Mage_Sales_Model_Quote
     */
    protected function _getQuote()
    {
        //if (!$this->_quote) {
        //    $this->_quote = $this->_getCheckoutSession()->getQuote();
        //}
        //return $this->_quote;

        //Always get the latest quote
        return $this->_getCheckoutSession()->getQuote();
    }

    public function createOrderForBackend($quote)
    {
        $incrementId = $this->_createOrderForBackend($quote->getId());
        $order       = Mage::getModel('sales/order')->load($incrementId, 'increment_id');
        $orderId     = $order->getId();
        return $this->_redirectUrl($this->_generateBackendUrl($orderId));
    }

    protected function _generateBackendUrl($orderId)
    {
        $url = Mage::helper('adminhtml')->getUrl('adminhtml/sales_order/view');
        $url = str_replace('view/', "view/order_id/$orderId/", $url);
        return $url;
    }

    protected function _createOrderForBackend($quoteId)
    {
        // Trigger capture method to set transaction data and place order
        $quote = Mage::getModel('sales/quote')->load($quoteId);
        $quote->collectTotals();
        $service = Mage::getModel('sales/service_quote', $quote);
        $service->submitAll();
        $quote->save();

        /* Get Order Id */
        $incrementId = $quote->getData('reserved_order_id');

        return $incrementId;
    }
}