<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/
class MageBinary_BinaryPay_Helper_Data extends Mage_Payment_Helper_Data
{
    /**
     * Convert under_score type array's keys to camelCase type array's keys
     * @param   array   $array          array to convert
     * @param   array   $arrayHolder    parent array holder for recursive array
     * @return  array   camelCase array
     */
    public function camelize($array, $arrayHolder = array())
    {
        $camelCaseArray = !empty($arrayHolder) ? $arrayHolder : array();
        foreach ($array as $key => $val) {
            $newKey = @explode('_', $key);
            array_walk($newKey, create_function('&$v', '$v = ucwords($v);'));
            $newKey = @implode('', $newKey);
            $newKey{0} = strtolower($newKey{0});
            if (!is_array($val)) {
                $camelCaseArray[$newKey] = $val;
            } else {
                $camelCaseArray[$newKey] = $this->camelize($val, $camelCaseArray[$newKey]);
            }
        }
        return $camelCaseArray;
    }
    /**
     * Convert camelCase type array's keys to under_score+lowercase type array's keys
     * @param   array   $array          array to convert
     * @param   array   $arrayHolder    parent array holder for recursive array
     * @return  array   under_score array
     */
    public function underscoreArray($array, $arrayHolder = array())
    {
        $underscoreArray = !empty($arrayHolder) ? $arrayHolder : array();
        foreach ($array as $key => $val) {
            $newKey = preg_replace('/[A-Z]/', '_$0', $key);
            $newKey = strtolower($newKey);
            $newKey = ltrim($newKey, '_');
            if (!is_array($val)) {
                $underscoreArray[$newKey] = $val;
            } else {
                $underscoreArray[$newKey] = $this->underscoreArray($val, $underscoreArray[$newKey]);
            }
        }
        return $underscoreArray;
    }

    /**
    * Translates a camel case string into a string with
    * underscores (e.g. firstName -> first_name)
    *
    * @param string $str String in camel case format
    * @return string $str Translated into underscore format
    */
    public function underscore($str)
    {
        $str[0] = strtolower($str[0]);
        $func = create_function('$c', 'return "_" . strtolower($c[1]);');
        return preg_replace_callback('/([A-Z])/', $func, $str);
    }

    public function transactionInformationFormatter($response)
    {
        $filters = array('resultCode', 'statusCode');

        if (!is_array($response)) {
            return $response;
        }

        foreach ($response as $key => $value) {
            // If the data should not be showed to customers
            // unset it from data array
            if (in_array($key, $filters)) {
                unset($response[$key]);
                continue;
            }

            if (strpos($key, '_')) {
                if (version_compare(PHP_VERSION, '5.4.32') >= 0) {
                    $newKey = str_replace('_', ' ', ucwords($key, '_'));
                } else {
                    $newKey = str_replace('_', ' ', $key);
                }
                $response[$newKey] = $response[$key];
                unset($response[$key]);
            }
        }

        return $response;
    }

    /**
     * Unset all data that should not be displayed in frontend
     * @param  array $args - additionalData
     * @return array
     */
    public function unsetPaymentInfo(&$args, $otherKeys = array())
    {
        $keys = array_merge(array('accountId', 'returnUrl', 'userAgent'), $otherKeys);
        foreach ($keys as $key) {
            if (isset($args[$key])) {
                unset($args[$key]);
            }
        }
    }

    /**
     * arrayFlatten converts multidimentional array to array
     * @param  array $array array to convert
     * @return array single level array
     */
    public function arrayFlatten($array)
    {
        if (!is_array($array)) {
            return false;
        }
        $result = array();
        foreach ($array as $key => $value) {
            if (is_array($value)) {
                $result = array_merge($result, $this->arrayFlatten($value));
            } else {
                $result[$key] = $value;
            }
        }
        return $result;
    }


    public function getLastMonthOrderQuantities()
    {
        $gateway    = Mage::getModel('magebinary_binarypay/directpost')->getGateway();
        $now        = $this->getCurrentStoreTime();
        $to         = date(DATE_ATOM, $now->getTimestamp());
        $from       = date(DATE_ATOM, strtotime('-1 month', $to));
        $orderSize  = Mage::getResourceModel('sales/order_item_collection')
                            ->addAttributeToSelect('order_id')
                            ->addAttributeToSelect('created_at')
                            ->addAttributeToFilter('created_at', array('from' => $from, 'to' => $to))
                            ->getSize();

        return $gateway->validateOrder($orderSize);
    }

    public function getCurrentStoreTime()
    {
       $timeZone = Mage::getStoreConfig('general/locale/timezone');
       $datetime = Zend_Date::now();

       $datetime->setLocale(Mage::getStoreConfig(Mage_Core_Model_Locale::XML_PATH_DEFAULT_LOCALE))
                ->setTimezone($timeZone);
       return $datetime;
    }

        /**
     * Prepare array with information about used product qty and product stock item
     * result is:
     * array(
     *  $productId  => array(
     *      'qty'   => $qty,
     *      'item'  => $stockItems|null
     *  )
     * )
     * @param array $relatedItems
     * @return array
     */
    public function getProductsQty($relatedItems)
    {
        $items = array();
        foreach ($relatedItems as $item) {
            $productId  = $item->getProductId();
            if (!$productId) {
                continue;
            }
            $children = $item->getChildrenItems();
            if ($children) {
                foreach ($children as $childItem) {
                    $this->_addItemToQtyArray($childItem, $items);
                }
            } else {
                $this->_addItemToQtyArray($item, $items);
            }
        }
        return $items;
    }

    /**
     * Adds stock item qty to $items (creates new entry or increments existing one)
     * $items is array with following structure:
     * array(
     *  $productId  => array(
     *      'qty'   => $qty,
     *      'item'  => $stockItems|null
     *  )
     * )
     *
     * @param Mage_Sales_Model_Quote_Item $quoteItem
     * @param array &$items
     */
    protected function _addItemToQtyArray($quoteItem, &$items)
    {
        $productId = $quoteItem->getProductId();
        if (!$productId)
            return;
        if (isset($items[$productId])) {
            $items[$productId]['qty'] += $quoteItem->getTotalQty();
        } else {
            $stockItem = null;
            if ($quoteItem->getProduct()) {
                $stockItem = $quoteItem->getProduct()->getStockItem();
            }
            $items[$productId] = array(
                'item' => $stockItem,
                'qty'  => $quoteItem->getTotalQty()
            );
        }
    }

    /**
     * Prepare array($productId=>$qty) based on array($productId => array('qty'=>$qty, 'item'=>$stockItem))
     *
     * @param array $items
     */
    protected function _prepareProductQtys($items)
    {
        $qtys = array();
        foreach ($items as $productId => $item) {
            if (empty($item['item'])) {
                $stockItem = Mage::getModel('cataloginventory/stock_item')->loadByProduct($productId);
            } else {
                $stockItem = $item['item'];
            }
            $canSubtractQty = $stockItem->getId() && $stockItem->canSubtractQty();
            if ($canSubtractQty && Mage::helper('catalogInventory')->isQty($stockItem->getTypeId())) {
                $qtys[$productId] = $item['qty'];
            }
        }
        return $qtys;
    }

    public function isStockAvailable($items)
    {
        $qtys = $this->_prepareProductQtys($items);
        $stockResource = Mage::getResourceModel('cataloginventory/stock');
        $stockInfo = $stockResource->getProductsStock(Mage::getSingleton('cataloginventory/stock'), array_keys($qtys), true);
        $item = Mage::getModel('cataloginventory/stock_item');
        foreach ($stockInfo as $itemInfo) {
            $item->setData($itemInfo);
            if (!$item->checkQty($qtys[$item->getProductId()])) {
                Mage::throwException(Mage::helper('cataloginventory')->__('Not all products are available in the requested quantity'));
            }
        }
    }

    /**
     * sendCreateOrderFailedEmail
     *
     * @param  Mage_Sales_Model_Quote $quote
     * @param  Array $paymentResponse
     * @param  Exception $exception
     * @return $this
     */
    public function sendCreateOrderFailedEmail($quote, $message, $checkoutType = 'onepage')
    {
        $session = Mage::getSingleton('checkout/session');
        $translate = Mage::getSingleton('core/translate');
        /* @var $translate Mage_Core_Model_Translate */
        $translate->setTranslateInline(false);

        $mailTemplate = Mage::getModel('core/email_template');
        /* @var $mailTemplate Mage_Core_Model_Email_Template */

        $template = Mage::getStoreConfig('checkout/payment_failed/template', $quote->getStoreId());

        $copyTo = $this->_getEmails('checkout/payment_failed/copy_to', $quote->getStoreId());
        $copyMethod = Mage::getStoreConfig('checkout/payment_failed/copy_method', $quote->getStoreId());
        if ($copyTo && $copyMethod == 'bcc') {
            $mailTemplate->addBcc($copyTo);
        }

        $_reciever = Mage::getStoreConfig('checkout/payment_failed/reciever', $quote->getStoreId());
        $sendTo = array(
            array(
                'email' => Mage::getStoreConfig('trans_email/ident_'.$_reciever.'/email', $quote->getStoreId()),
                'name'  => Mage::getStoreConfig('trans_email/ident_'.$_reciever.'/name', $quote->getStoreId())
            )
        );

        if ($copyTo && $copyMethod == 'copy') {
            foreach ($copyTo as $email) {
                $sendTo[] = array(
                    'email' => $email,
                    'name'  => null
                );
            }
        }
        $shippingMethod = '';
        if ($shippingInfo = $quote->getShippingAddress()->getShippingMethod()) {
            $data = explode('_', $shippingInfo);
            $shippingMethod = $data[0];
        }

        $paymentMethod = '';
        if ($paymentInfo = $quote->getPayment()) {
            $paymentMethod = $paymentInfo->getMethod();
        }

        $items = '';
        foreach ($quote->getAllVisibleItems() as $_item) {
            /* @var $_item Mage_Sales_Model_Quote_Item */
            $items .= $_item->getProduct()->getName() . '  x '. $_item->getQty() . '  '
                . $quote->getStoreCurrencyCode() . ' '
                . $_item->getProduct()->getFinalPrice($_item->getQty()) . "\n";
        }
        $total = $quote->getStoreCurrencyCode() . ' ' . $quote->getGrandTotal();
        $customerHelper = Mage::helper('customer');
        // it is better to get the customer name from the quote because the customer session lost when failed to placing an order
        $fullCustomerName = method_exists($customerHelper, 'getFullCustomerName') ? $customerHelper->getFullCustomerName($quote)
            : $customerHelper->getCustomerName();

        foreach ($sendTo as $recipient) {
            $mailTemplate->setDesignConfig(array('area'=>'frontend', 'store'=>$quote->getStoreId()))
                ->sendTransactional(
                $template,
                Mage::getStoreConfig('checkout/payment_failed/identity', $quote->getStoreId()),
                $recipient['email'],
                $recipient['name'],
                    array(
                        'reason'          => $message,
                        'checkoutType'    => $checkoutType,
                        'dateAndTime'     => Mage::app()->getLocale()->date(),
                        'customer'        => $fullCustomerName,
                        'customerEmail'   => $quote->getCustomerEmail(),
                        'billingAddress'  => $quote->getBillingAddress(),
                        'shippingAddress' => $quote->getShippingAddress(),
                        'shippingMethod'  => Mage::getStoreConfig('carriers/' . $shippingMethod . '/title'),
                        'paymentMethod'   => Mage::getStoreConfig('payment/' . $paymentMethod . '/title'),
                        'items'           => nl2br($items),
                        'total'           => $total,
                        'response'        => json_encode($session->getResponse())
                    )
            );
        }

        $translate->setTranslateInline(true);
        return $this;
    }

    protected function _getEmails($configPath, $storeId)
    {
        $data = Mage::getStoreConfig($configPath, $storeId);
        if (!empty($data)) {
            return explode(',', $data);
        }
        return false;
    }
}
