<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/
class MageBinary_BinaryPay_Model_Directpost extends MageBinary_BinaryPay_Model_Abstract
{
    protected $_code    = MageBinary_BinaryPay_Model_Config::PAYMARK_DIRECTPOST;

    public function saveToken($data)
    {
        $card       = new Varien_Object();
        /* Throw this to the lib */
        $cardKeys   = array('cardType', 'cardNumber', 'cardExpiry', 'cardHolder', 'cardToken', 'tokenReference');

        foreach ($cardKeys as $key) {
            if (isset($data[$key])) {
                $card->setData(Mage::helper('magebinary_binarypay')->underscore($key), $data[$key]);
            }
        }

        // validate data info
        if (isset($data['code']) && isset($data['message'])) {
            $code       = $data['code'];
            $message    = $data['message'];
            Mage::throwException("code: $code Message: $message");
        }

        parent::saveToken($card);
    }

    public function deleteTokenByReference($reference)
    {
        $customer   = $this->getCustomer();
        //Get the tokens of the current customer
        $tokens     = $customer->getPmSavedTokens();

        $this->getGateway()->delete(Mage::helper('magebinary_binarypay')->camelize($tokens[$reference]));

        parent::deleteTokenByReference($reference);
    }

    public function generateCreditCardTokenReference($cardInfo)
    {
        $cardNumber     = $cardInfo[BinaryPay::CARD_NUMBER];
        $cardExpiry     = $cardInfo[BinaryPay::CARD_EXPIRY];
        $data           = strlen($cardExpiry) === 4 ? $cardNumber . $cardExpiry : '';
        return empty($data) ? false : md5(Mage::helper('core')->encrypt($data));
    }
}