<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 - 2019 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/

abstract class MageBinary_BinaryPay_Model_Method_Abstract extends Mage_Payment_Model_Method_Abstract
{
    /**
     * _getReferenceNumber - get next order Id by last order id, to fix webpayment multiple increment id number bugs
     * @return Integer
     */
    protected function _getReferenceNumber()
    {
        $reserveOrderId = Mage::getSingleton('checkout/session')->getQuote()->reserveOrderId()->getReservedOrderId();
        if (!$reserveOrderId) {
            Mage::throwException('Can not get reserved order Id currently. Please try to checkout again.');
        }
        Mage::getSingleton('checkout/session')->getQuote()->reserveOrderId()->save();
        return $reserveOrderId;
    }

        /**
     * Check whether payment method can be used
     *
     * @param Mage_Sales_Model_Quote|null $quote
     *
     * @return bool
     */
    public function isAvailable($quote = null)
    {
        return Mage::getModel($this->_modelName)->getPaymentMethodConfig('active');
    }

    /**
     * Retrieve quote object
     *
     * @return Mage_Sales_Model_Quote
     */
    protected function _getQuote()
    {
        return $this->getInfoInstance()->getQuote();
    }

    /**
     * Is the payment on hold
     *
     * @return boolean
     */
    protected function _isPaymentOnHold()
    {
        return Mage::getSingleton('checkout/session')->getIsPending();
    }

    /**
     * _getQuoteProducts
     * @return [type]
     */
    protected function _getQuoteProducts()
    {
        $items = $this->_getQuote()->getAllVisibleItems();
        $products = [];
        foreach ($items as $_item) {
            $_product = $_item->getProduct();
            $productItem = [
                'name'          => $_product->getName(),
                'price' => [
                    'amount'    => round($_item->getPriceInclTax(),2),
                    'currency'  => $this->getCurrencyCode()
                ],
                'sku'           => $_product->getSku(),
                'quantity'      => $_item->getQty(),
                'taxIncluded'   => 1
            ];
            array_push($products, $productItem);
        }
        return $products;
    }

    /**
     * _getShippingData
     * @return [type]
     */
    protected function _getShippingData()
    {
        $shipping = $this->_getQuote()->getShippingAddress();
        $shippingDetail = [
            'carrier' => $shipping->getShippingMethod(),
            'price' => [
                'amount' => round($shipping->getShippingTaxable(),2),
                'currency' => $this->getCurrencyCode()
            ],
            'taxIncluded' => ($shipping->getShippingTaxable()) ? 1 : 0
        ];
        return $shippingDetail;
    }

    /**
     * _getAmount
     * @param  boolean $isBackend
     * @return float
     */
    public function getAmount($isBackend = false)
    {
        if ($isBackend) {
            $amount = Mage::getSingleton('adminhtml/session_quote')->getQuote()->getGrandTotal();
            return $amount;
        }
        $amount    = $this->_getQuote()->getGrandTotal();
        return $amount;
    }

    /**
     * @return Mage_Checkout_Model_Session
     */
    public function getCheckoutSession()
    {
        return Mage::getSingleton('checkout/session');
    }

    /**
     * @return string
     */
    public function getCurrencyCode()
    {
        return Mage::app()->getStore()->getCurrentCurrencyCode();
    }

    /**
     * isLoggedIn
     * @return boolean
     */
    public function isLoggedIn()
    {
        return Mage::getSingleton('customer/session')->isLoggedIn();
    }

    public function getCustomer()
    {
        return Mage::getSingleton('customer/session')->getCustomer();
    }
}