<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/

class MageBinary_BinaryPay_Model_Method_Flexilong extends MageBinary_BinaryPay_Model_Method_Abstract
{
    protected $_formBlockType               = 'magebinary_binarypay/flexilong_form';
    protected $_infoBlockType               = 'magebinary_binarypay/flexilong_info';
    protected $_code                        = MageBinary_BinaryPay_Model_Config::QCARD_FLEXILONG;
    protected $_modelName                   = 'magebinary_binarypay/flexilong';
    protected $_isGateway                   = false;
    protected $_canAuthorize                = false;
    protected $_canCapture                  = true;
    protected $_canRefund                   = false;
    protected $_canRefundInvoicePartial     = false;
    protected $_canFetchTransactionInfo     = true;
    protected $_canUseCheckout              = true;
    protected $_canVoid                     = false;

    const DECLINED = 'DECLINED';
    const IN_PROGRESS = 'IN_PROGRESS';
    const ERROR = 'ERROR';
    const CANCEL = 'CANCELLED';
    const APPROVED = 'APPROVED';


    public function assignData($data)
    {
        $session = Mage::getSingleton('checkout/session');

        if (!($data instanceof Varien_Object)) {
            $data = new Varien_Object($data);
        }

        $session->setFlexiTerm($data->getFlexiTerm());

        $this->getInfoInstance()
             ->setFlexiTerm($data->getFlexiTerm())
             ->setPoNumber($data->getPurchaseOrder());

        return $this;
    }

    public function getCheckoutRedirectUrl()
    {
        $purchaseUrl    = $this->getPurchaseUrl();
        return $purchaseUrl;
    }

    protected function _getAmount($isBackend = false)
    {
        if ($isBackend) {
            $amount = Mage::getSingleton('adminhtml/session_quote')->getQuote()->getGrandTotal();
            return $amount;
        }
        $amount    = $this->_getQuote()->getGrandTotal();
        return $amount;
    }

    public function getPurchaseUrl($isBackend = false)
    {
        try {
            (!$isBackend) ? $info = $this->getInfoInstance() : $flexiTerm = Mage::getModel('core/cookie')->get('flexiTerm');

            $api         = Mage::getModel($this->_modelName);
            $gateway     = $api->getGateway();

            $apiKey      = $api->getPaymentMethodConfig('qcard_payment_apikey', true);

            // Mage::getModel('core/session')->setNextIncrement($this->_getReferenceNumber());

            // encrypt referencenumber
            $reference   = $this->_getReferenceNumber();
            $amount      = $this->getAmount($isBackend);

            // API_KEY
            $payment = array(
                BinaryPay::API_KEY      => $apiKey,
                BinaryPay::REFERENCE    => $reference,
                BinaryPay::AMOUNT       => $amount,
                BinaryPay::PRODUCT_CODE => (isset($info)) ? $info->getFlexiTerm() : $flexiTerm,
                BinaryPay::RETURN_URL   => Mage::getUrl('binarypay/qcard/return'),
                BinaryPay::REDIRECT     => true,
            );
            $purchase = $gateway->purchase($payment);
            $purchaseUrl = $this->_getPurchaseUrl($purchase);

        }  catch (BinaryPay_Exception $e) {
            Mage::throwException($e->getMessage());
        } catch (Exception $e) {
            $message = $e->getMessage();
            if (empty($message)) {
                Mage::throwException('Something massively went wrong. Please try again. If the problem still exists, please contact us');
            } else {
                Mage::throwException($e->getMessage());
            }
            Mage::log($e->getTrace(), null, 'BinaryPay.log', true);
        }
        return $purchaseUrl;
    }

    private function _getPurchaseUrl($purchase)
    {
        // Get Purchase Url from response
        if (isset($purchase['payment_url'])) {
            $purchaseUrl = $purchase['payment_url'];
        } else {
            Mage::throwException('Oops! Some errors has occurred, we can not redirect you to the Qcard. Please try again later.');
        }

        return $purchaseUrl;
    }

    public function capture(Varien_Object $payment, $amount)
    {
        $session   = Mage::getSingleton('checkout/session');

        // Purchased Without Saved Token
        $response   = $session->getQcardResponse();
        $order      = $payment->getOrder();
        $response   = Mage::helper('magebinary_binarypay')->arrayFlatten($response);
        $formatted  = Mage::helper('magebinary_binarypay')->transactionInformationFormatter($response);

        $status = Mage_Sales_Model_Order::STATE_PROCESSING;

        $order->setState(Mage_Sales_Model_Order::STATE_PROCESSING, true, '')->save();

        if ($this->_isPaymentOnHold()) {
            Mage::getSingleton('checkout/session')->unsIsPending();
            $payment->setIsTransactionPending(true);
        }

        $payment->setStatus(self::STATUS_APPROVED);
        $payment->setLastTransId($response['merchant_transaction_id'])
                ->setTransactionId($response['merchant_transaction_id'])
                ->setAdditionalData(serialize($formatted));

        // Prevent Session Abuse
        $session->unsQcardResponse();

        return $this;
    }

    public function fetchTransactionInfo(Mage_Payment_Model_Info $payment, $transactionId)
    {
        try {
            $api        = Mage::getModel($this->_modelName);
            $gateway    = $api->getGateway();

            $order          = $payment->getOrder();
            $paymentObject  = $order->getPayment();

            $transctionOject = Mage::getModel('sales/order_payment_transaction')->setOrderPaymentObject($paymentObject);
            $transaction = $transctionOject->loadByTxnId($transactionId);

            $additionalData = unserialize($paymentObject->getAdditionalData());
            $processNo      = $additionalData['processNo'];

            $request = array(
                BinaryPay::API_KEY        => $api->getPaymentMethodConfig('qcard_status_apikey', true),
                BinaryPay::TRANSACTION_ID => $transactionId,
                BinaryPay::PROCESS_NO     => $processNo
            );

            $response = $gateway->getTransaction($request);
            $response = $gateway->arrayFlatten($response);

            $order->getPayment()->setAdditionalData(serialize($response));
            if ($transaction['statusDesc'] === BinaryPay::STATUS_APPROVED) {
                $order->setState(
                    Mage_Sales_Model_Order::STATE_PROCESSING, true
                )->save();
            }

        } catch (BinaryPay_Exception $e) {
            Mage::throwException($e->getMessage());
        } catch (Exception $e) {
            Mage::throwException('Something massively went wrong. Please try again. If the problem still exists, please contact us');
            Mage::log($e->getTrace(), null, 'BinaryPay.log', true);
        }
        return $response;
    }
}