<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 - 2019 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 - 2019 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/
class MageBinary_BinaryPay_Model_Method_Latitudepay extends MageBinary_BinaryPay_Model_Method_Genoapay
{
    const LATITUDE_ENABLED                  = "payment/latitudepay/active";
    const LATITUDE_CURRENCY                 = "payment/latitudepay/currency";
    protected $_formBlockType               = 'magebinary_binarypay/latitudepay_form';
    protected $_infoBlockType               = 'magebinary_binarypay/latitudepay_info';

    protected $_code                        =  MageBinary_BinaryPay_Model_Config::LATITUDEPAY;
    protected $_modelName                   = 'magebinary_binarypay/latitudepay';
    protected $_returnUrl                   = 'binarypay/latitudepay/return';

     /**
     * _getPurchaseUrl
     * @param  array $purchase
     * @return string
     */
    private function _getPurchaseUrl($purchase)
    {
        // Get Purchase Url from response
        if (isset($purchase['paymentUrl'])) {
            $purchaseUrl = $purchase['paymentUrl'];
        } else {
            Mage::throwException('Oops! Some errors has occurred, we can not redirect you to the LatitudePay. Please try again later.');
        }
        return $purchaseUrl;
    }

    public function setSesionReferense($reference, $response)
    {
        Mage::getSingleton('checkout/session')->setLatitudepayReference($reference);
        Mage::getSingleton('checkout/session')->setLatitudepayToken($response['token']);
    }

    /**
     * Check whether payment method is applicable to quote
     * Purposed to allow use in controllers some logic that was implemented in blocks only before
     *
     * @param Mage_Sales_Model_Quote $quote
     * @param int|null $checksBitMask
     * @return bool
     */
    public function isApplicableToQuote($quote, $checksBitMask)
    {
        return $this->isAvailable($quote) && Mage::getStoreConfig(self::LATITUDE_ENABLED, $store) &&  (Mage::getStoreConfig(self::LATITUDE_CURRENCY) == Mage::app()->getStore()->getCurrentCurrencyCode());
    }
}