<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/

class MageBinary_BinaryPay_Model_Method_Onlineeftpos extends Mage_Payment_Model_Method_Abstract
{
    protected $_formBlockType               = 'magebinary_binarypay/onlineeftpos_form';
    protected $_infoBlockType               = 'magebinary_binarypay/onlineeftpos_info';
    protected $_code                        = MageBinary_BinaryPay_Model_Config::PAYMARK_ONLINEEFTPOS;
    protected $_isGateway                   = true;
    protected $_canAuthorize                = false;
    protected $_canCapture                  = true;
    protected $_canRefund                   = true;
    protected $_canRefundInvoicePartial     = true;
    protected $_canFetchTransactionInfo     = true;
    protected $_canUseCheckout              = true;
    protected $_canVoid                     = true;

    public function assignData($data)
    {
        if (!($data instanceof Varien_Object)) {
            $data = new Varien_Object($data);
        }

        $this->getInfoInstance()
             ->setMobile($data->getMobile())
             ->setPoNumber($data->getPurchaseOrder())
             ->setDescription($data->getDescription())
             ->setIsMobileSaved($data->getIsMobileSaved())
             ->setSavedMobile($data->getSavedMobile());
        return $this;
    }

    public function isAvailable($quote = null)
    {
        return Mage::getModel('magebinary_binarypay/onlineeftpos')->getPaymentMethodConfig('active');
    }

    public function validate()
    {
        return $this;
    }

    public function capture(Varien_Object $payment, $amount)
    {
        $session        = Mage::getSingleton('core/session');
        $api            = Mage::getModel('magebinary_binarypay/onlineeftpos');
        $order          = $payment->getOrder();
        $gateway        = $api->getGateway();
        $mobile         = $payment->getMobile();
        $mobileNumber   = empty($mobile) ? $payment->getSavedMobile() : $mobile;
        $description    = $payment->getDescription();
        $purchaseOrder  = array(
            'purchaseOrder' => $payment->getPoNumber()
        );

        $transactionData = array(
            BinaryPay::MOBILENUMBER => $mobileNumber,
            BinaryPay::ACCOUNTID    => $api->getPaymentMethodConfig('account_id', true),
            BinaryPay::RETURN_URL   => Mage::getUrl('binarypay/eftpos/return'),
            // BinaryPay::RETURN_URL   => "http://dev.magebinary.com:7788/binarypay/eftpos/return/",
            BinaryPay::MERCHANT_URL => Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_LINK),
            BinaryPay::AMOUNT       => $amount,
            BinaryPay::CURRENCY     => Mage::app()->getStore()->getCurrentCurrencyCode(),
            BinaryPay::DESCRIPTION  => $description,
            BinaryPay::REFERENCE    => $order->getIncrementId(),
            BinaryPay::USER_AGENT   => Mage::helper('core/http')->getHttpUserAgent(),
            BinaryPay::IP           => Mage::helper('core/http')->getRemoteAddr()
        );

        try {
            if ($payment->getIsMobileSaved()) {
                $tokenRef   = $api->generateTokenReference($mobileNumber);
                $data = array(
                    'token_reference'   => $tokenRef,
                    'mobile_number'     => $mobileNumber
                );
                $api->saveToken($data);
            }

            $response = $gateway->purchase($transactionData);
            $session->setIncrementId($order->getIncrementId());

            $payment->setStatus(self::STATUS_APPROVED)
                    ->setLastTransId($response['id'])
                    ->setTransactionId($response['id']);
            $poNumber = $payment->getPoNumber();
            if (!empty($poNumber)) {
                $response = array_merge($response, $purchaseOrder);
            }

            $response = $gateway->arrayFlatten($response);
            $response = $gateway->clearify($response);
            $payment->setAdditionalData(serialize($response));

        } catch (BinaryPay_Exception $e) {
            Mage::throwException($e->getMessage());
        } catch (Exception $e) {
            Mage::throwException('Something massively went wrong. Please try again. If the problem still exists, please contact us');
            Mage::log($e->getTrace(), null, 'paymark.log', true);
        }
        return $this;
    }

    /**
     * Refund specified amount for payment
     *
     * @param Varien_Object $payment
     * @param float $amount
     *
     * @return Mage_Payment_Model_Abstract
     */
    public function refund(Varien_Object $payment, $amount)
    {
        $api        = Mage::getModel('magebinary_binarypay/onlineeftpos');
        $gateway    = $api->getGateway();

        $order      = $payment->getOrder();
        $refund     = array(
           BinaryPay::ACCOUNTID   => $api->getPaymentMethodConfig('account_id', true),
           BinaryPay::AMOUNT      => $amount,
           BinaryPay::DESCRIPTION => '',
           BinaryPay::ORIGIN_TRANSACTION_ID   => $payment->getLastTransId(),
           BinaryPay::REFUND_ID   => $order->getIncrementId(),
           BinaryPay::USER_AGENT  => Mage::helper('core/http')->getHttpUserAgent(),
           BinaryPay::IP          => Mage::helper('core/http')->getRemoteAddr()
        );

        try {
            $refundTranscation = $gateway->refund($refund);
            $payment->setTransactionId($refundTranscation['id']);
            $shouldCloseCaptureTransaction = $payment->getOrder()->canCreditmemo() ? 0 : 1;
            $payment->setShouldCloseParentTransaction($shouldCloseCaptureTransaction)
                    ->setTransactionAdditionalInfo(serialize($refundTranscation), $payment->getTransactionId());
        }  catch (BinaryPay_Exception $e) {
            Mage::throwException($e->getMessage());
        } catch (Exception $e) {
            Mage::throwException('Something massively went wrong. Please try again. If the problem still exists, please contact us');
            Mage::log($e->getTrace(), null, 'paymark.log', true);
        }
        return $this;
    }


    public function getOrderPlaceRedirectUrl()
    {
        return Mage::getUrl('binarypay/eftpos/start');
    }

    public function fetchTransactionInfo(Mage_Payment_Model_Info $payment, $transactionId)
    {
        $api            = Mage::getModel('magebinary_binarypay/onlineeftpos');
        $gateway        = $api->getGateway();
        $order          = $payment->getOrder();
        $paymentObject  = $order->getPayment();

        // Get txn type
        $transctionOject = Mage::getModel('sales/order_payment_transaction')->setOrderPaymentObject($paymentObject);
        $transaction = $transctionOject->loadByTxnId($transactionId);
        $transactionType = $transaction->getTxnType();

        try {
            // Get transaction depends on the txn type
            if ($transactionType === 'refund') {
                $transaction = $gateway->getTransaction($transactionId, true);
            } else {
                $transaction = $gateway->getTransaction($transactionId);
            }

            $transaction = $gateway->arrayFlatten($transaction);
            $transaction = $gateway->clearify($transaction);
            $order->getPayment()->setAdditionalData(serialize($transaction));
            if ($transaction['status'] === BinaryPay::STATUS_AUTHORISED) {
                $order->setState(Mage_Sales_Model_Order::STATE_PROCESSING, true)->save();
            }
        } catch (BinaryPay_Exception $e) {
            Mage::throwException($e->getMessage());
        } catch (Exception $e) {
            Mage::throwException('Something massively went wrong. Please try again. If the problem still exists, please contact us');
            Mage::log($e->getTrace(), null, 'paymark.log', true);
        }
        return $transaction;
    }

}