<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 - 2019 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 - 2019 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/
class MageBinary_BinaryPay_Model_Method_Polipay extends MageBinary_BinaryPay_Model_Method_Abstract
{
    protected $_formBlockType               = 'magebinary_binarypay/polipay_form';
    protected $_infoBlockType               = 'magebinary_binarypay/polipay_info';
    protected $_code                        =  MageBinary_BinaryPay_Model_Config::POLIPAY;
    protected $_modelName                   = 'magebinary_binarypay/polipay';
    protected $_returnUrl                   = 'binarypay/polipay/return';
    protected $_failureURL                  = 'binarypay/polipay/return';
    protected $_notificationURL             = 'binarypay/polipay/return';
    protected $_cancellationURL             = 'binarypay/polipay/return';
    protected $_isGateway                   = false;
    protected $_canAuthorize                = false;
    protected $_canCapture                  = true;
    protected $_canRefund                   = false;
    protected $_canRefundInvoicePartial     = false;
    protected $_canFetchTransactionInfo     = true;
    protected $_canUseCheckout              = true;
    protected $_canVoid                     = false;

    const DEFAULT_VALUE                     = 'NONE';
    const SUCCESSFUL                        = 'Completed';
    const DECLINED                          = 'DECLINED';
    const CANCELLED                         = 'Cancelled';
    const INPROGRESS                        = 'InProcess';
    const UNKNOWN                           = 'Unknown';
    const FAILED                            = 'Failed';
    Const TIMEOUT                           = 'TimedOut';
    const RECEIPTUNVERIFIED                 = 'ReceiptUnverified';

    /**
     * getCheckoutRedirectUrl
     * @return string
     */
    public function getCheckoutRedirectUrl()
    {
        $purchaseUrl    = $this->getPurchaseUrl();
        return $purchaseUrl;
    }

    /**
     * assignData
     * @param  Varien_Object | Array $data
     * @return MageBinary_BinaryPay_Model_Method_Genopay
     */
    public function assignData($data)
    {
        $session = Mage::getSingleton('checkout/session');

        if (!($data instanceof Varien_Object)) {
            $data = new Varien_Object($data);
        }

        $this->getInfoInstance()
             ->setPoNumber($data->getPurchaseOrder());

        return $this;
    }

    /**
     * Capture payment
     *
     * @param Varien_Object $payment
     * @param float $amount
     *
     * @return MageBinary_BinaryPay_Model_Method_Polipay
     */
    public function capture(Varien_Object $payment, $amount)
    {
        $session = Mage::getSingleton('checkout/session');

        $gateway    = Mage::getModel($this->_modelName)->getGateway();
        $response   = $gateway->clearify($session->getResponse());

        $payment->setLastTransId($response['TransactionID'])
                ->setTransactionId($response['TransactionID'])
                ->setAdditionalData(serialize($response));
        return $this;
    }

    /**
     * Fetch transaction info
     *
     * @param Mage_Payment_Model_Info $payment
     * @param string $transactionId
     * @return array
     */
    public function fetchTransactionInfo(Mage_Payment_Model_Info $payment, $transactionId)
    {
        $api        = Mage::getModel($this->_modelName);
        $gateway    = $api->getGateway();

        try {
            $additionalInformation = unserialize($payment->getAdditionalData());
            $transcation = $gateway->retrieve(array(
                BinaryPay::PURCHASE_TOKEN => $additionalInformation['token']
            ));

        } catch (BinaryPay_Exception $e) {
            Mage::throwException($e->getMessage());
        } catch (Exception $e) {
            Mage::throwException('Something massively went wrong. Please try again. If the problem still exists, please contact us');
            Mage::log($e->getTrace(), null, 'BinaryPay.log', true);
        }
        return $transcation;
    }

    public function getPurchaseUrl()
    {
        try {
            /**
             * TODO: Support backend payment
             * @var boolean
             */
            $isBackend  = false;

            $api        = Mage::getModel($this->_modelName);
            $gateway    = $api->getGateway();

            $reference          = $this->_getReferenceNumber();
            $amount             = $this->getAmount($isBackend);

            $payment = array(
                BinaryPay::REFERENCE                => (string) $reference,
                BinaryPay::REFERENCEFORMAT          => '4',
                BinaryPay::AMOUNT                   => $amount,
                BinaryPay::CURRENCY                 => $this->getCurrencyCode() ?: self::DEFAULT_VALUE,
                BinaryPay::RETURN_URL               => Mage::getUrl($this->_returnUrl),
                BinaryPay::MERCHANT_URL             => Mage::getBaseUrl(),
                BinaryPay::FAILURE_URL              => Mage::getUrl($this->_failureURL),
                BinaryPay::MERCHANT_CODE            => (string) $reference,
                // BinaryPay::NOTIFICATION_URL         => Mage::getUrl($this->_notificationURL),
                BinaryPay::CANCELLATION_URL         => Mage::getUrl($this->_cancellationURL)
            );

            $response   = $gateway->purchase($payment);


            $parts = parse_url($response['NavigateURL']);
            parse_str($parts['query'], $query);



            Mage::getSingleton('checkout/session')->setPolipayReference($reference);
            Mage::getSingleton('checkout/session')->setPolipayToken($query['Token']);

            $decodeurl = $this->_getPurchaseUrl($response);
            $purchaseUrl = urldecode($decodeurl);


        } catch (BinaryPay_Exception $e) {
            Mage::throwException($e->getMessage());
        } catch (Exception $e) {
            $message = $e->getMessage() ?: 'Something massively went wrong. Please try again. If the problem still exists, please contact us';
            Mage::throwException($message);
            Mage::log($e->getTrace(), null, 'BinaryPay.log', true);
        }

        return $purchaseUrl;
    }

    /**
     * _getPurchaseUrl
     * @param  array $purchase
     * @return string
     */
    private function _getPurchaseUrl($purchase)
    {
        // Get Purchase Url from response
        if (isset($purchase['NavigateURL'])) {
            $purchaseUrl = $purchase['NavigateURL'];
        } else {
            Mage::throwException('Oops! Some errors has occurred, we can not redirect you to the Polipay. Please try again later.');
        }
        return $purchaseUrl;
    }

    protected function _getCustomerEmail()
    {
        return ($this->isLoggedIn()) ? $this->getCustomer()->getEmail() : $this->_getQuote()->getCustomerEmail();
    }

    public function _getFirstName()
    {
        return ($this->isLoggedIn()) ? $this->getCustomer()->getFirstname() : '';
    }

    public function _getSurname()
    {
        return ($this->isLoggedIn()) ? $this->getCustomer()->getLastname() : '';
    }
}