<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/

class MageBinary_BinaryPay_Model_Method_Webpayment extends MageBinary_BinaryPay_Model_Method_Directpost
{
    protected $_formBlockType               = 'magebinary_binarypay/webpayment_form';
    protected $_infoBlockType               = 'magebinary_binarypay/webpayment_info';
    protected $_code                        = MageBinary_BinaryPay_Model_Config::PAYMARK_WEBPAYMENT;
    protected $_isGateway                   = false;
    protected $_canAuthorize                = false;
    protected $_canCapture                  = true;
    protected $_canRefund                   = true;
    protected $_canRefundInvoicePartial     = true;
    protected $_canFetchTransactionInfo     = true;
    protected $_canUseCheckout              = true;
    protected $_canVoid                     = false;

    public function assignData($data)
    {
        if (!($data instanceof Varien_Object)) {
            $data = new Varien_Object($data);
        }
        $this->getInfoInstance()
             ->setCardToken($data->getCardToken())
             ->setPoNumber($data->getPurchaseOrder());

        return $this;
    }

    public function getCheckoutRedirectUrl()
    {
        $session        = Mage::getSingleton('checkout/session');
        $cardToken      = $this->getInfoInstance()->getCardToken();
        $purchaseUrl    = $this->getPurchaseUrl();

        // not using saved card, redirect to paymark webpayment
        if (!$cardToken) {
            return $purchaseUrl;
        }
    }

    public function isAvailable($quote = null)
    {
        return  Mage::getModel('magebinary_binarypay/webpayment')->getPaymentMethodConfig('active');
    }

    public function capture(Varien_Object $payment, $amount)
    {
        $session    = Mage::getSingleton('checkout/session');
        $cardToken  = $payment->getCardToken();

        $purchaseOrder = array(
            'purchaseOrder' => $payment->getPoNumber()
        );

        // Purchased With Saved Token
        if (!empty($cardToken)) {
            return parent::capture($payment, $amount);
        }

        // Purchased Without Saved Token
        $gateway    = Mage::getModel('magebinary_binarypay/webpayment')->getGateway();
        $response   = $gateway->clearify($session->getResponse());
        $poNumber = $payment->getPoNumber();
        if (!empty($poNumber)) {
            $response   = array_merge($response, $purchaseOrder);
        }

        $payment->setLastTransId($response['transactionId'])
                ->setTransactionId($response['transactionId'])
                ->setAdditionalData(serialize($response));
        return $this;
    }

    /**
     * Retrieve quote object
     *
     * @return Mage_Sales_Model_Quote
     */
    protected function _getQuote()
    {
        $quote = $this->getInfoInstance()->getQuote();
        return $quote;
    }

    public function validate()
    {
        return $this;
    }

    /**
     * _getReferenceNumber - get next order Id by last order id, to fix webpayment multiple increment id number bugs
     * @return Integer
     */
    protected function _getReferenceNumber()
    {
        $reserveOrderId = Mage::getSingleton('checkout/session')->getQuote()->reserveOrderId()->getReservedOrderId();
        if (!$reserveOrderId) {
            Mage::throwException('Can not get reserved order Id currently. Please try to checkout again.');
        }
        return $reserveOrderId;
    }

    public function getPurchaseUrl($isBackend = false)
    {
        $cmd                = $this->_cmd;
        $api                = Mage::getModel('magebinary_binarypay/webpayment');
        $gateway            = $api->getGateway();
        $accountId          = $api->getPaymentMethodConfig('account_id', true);
        $amount             = $this->_getAmount($isBackend);
        $reference          = $this->_getReferenceNumber();
        $particular         = $api->generateTranscationParticular();
        // $storeCard          = Mage::getSingleton('customer/session')->isLoggedIn() ? 1 : 0;
        // $returnUrl          = Mage::getUrl('binarypay/tokens/return');
        $returnUrl          = Mage::getModel('core/url')
                                ->setQueryParam('SID', Mage::getSingleton('core/session')->getEncryptedSessionId())
                                ->setQueryParam('QID', $this->_getQuote()->getId())
                                ->setQueryParam('CID', Mage::getSingleton('customer/session')->getCustomerId())
                                ->setQueryParam('VID', Mage::getModel('core/session')->getVisitorId())
                                ->setQueryParam('SCID', Mage::getSingleton('core/cookie')->get('frontend_cid'))
                                ->getUrl('binarypay/tokens/return');


        //Check if Customer loggedIn and StoreCard option is enabled SP-2567
        if (Mage::getSingleton('customer/session')->isLoggedIn() == 1 && $api->getPaymentMethodConfig('scard') == 1 ){
            $storeCard = 1;
        }
        else{
            $storeCard = 0;
        }

        $webpayment = array(
            BinaryPay::ACCOUNTID     => $accountId,
            BinaryPay::AMOUNT        => $amount,
            BinaryPay::REFERENCE     => $reference,
            BinaryPay::PARTICULAR    => $particular,
            BinaryPay::RETURN_URL    => $returnUrl,
            BinaryPay::STORE_CARD    => $storeCard,
            BinaryPay::DISPLAY_EMAIL => 0,
        );

        // Save particular for return validation
        Mage::getSingleton('checkout/session')->setParticular($particular);

        // Set particular in cookie when consumer place order in backend
        if (Mage::app()->getStore()->isAdmin()) {
            Mage::getSingleton('core/cookie')->set('particular', $particular);
        }

        try {
            $purchaseUrl = $gateway->purchase($webpayment);
            $this->_isValidPurchaseUrl($purchaseUrl);

        } catch (Mage_Core_Exception $e) {
            Mage::throwException($e->getMessage());
        } catch (BinaryPay_Exception $e) {
            Mage::throwException($e->getMessage());
        } catch (Exception $e) {
            if (!$e->getMessage()) {
                Mage::throwException('Something massively went wrong. Please try again. If the problem still exists, please contact us');
            }
            Mage::throwException($e->getMessage());
            Mage::log($e->getTrace(), null, 'BinaryPay.log', true);
        }
        return $purchaseUrl;
    }

     /**
     * Refund specified amount for payment
     *
     * @param Varien_Object $payment
     * @param float $amount
     *
     * @return Mage_Payment_Model_Abstract
     */
    public function refund(Varien_Object $payment, $amount)
    {
        $api        = Mage::getModel('magebinary_binarypay/webpayment');
        $gateway    = $api->getGateway();

        $order      = $payment->getOrder();
        $refund     = array(
             BinaryPay::ACCOUNTID               => $api->getPaymentMethodConfig('account_id', true),
             BinaryPay::TRANSACTION_ID          => $payment->getParentTransactionId(),
             BinaryPay::AMOUNT                  => $amount,
             BinaryPay::REFERENCE               => $order->getIncrementId(),
             BinaryPay::PARTICULAR              => $order->getIncrementId(),
             BinaryPay::EMAIL                   => $order->getCustomerEmail()
        );

        try {
            $refundTranscation = $gateway->refund($refund);
            $payment->setTransactionId($refundTranscation['transactionId']);
            $shouldCloseCaptureTransaction = $payment->getOrder()->canCreditmemo() ? 0 : 1;
            $payment->setShouldCloseParentTransaction($shouldCloseCaptureTransaction)
                    ->setTransactionAdditionalInfo(serialize($refundTranscation), $payment->getTransactionId());
        } catch (BinaryPay_Exception $e) {
            Mage::throwException($e->getMessage());
        } catch (Exception $e) {
            Mage::throwException('Something massively went wrong. Please try again. If the problem still exists, please contact us');
            Mage::log($e->getTrace(), null, 'paymark.log', true);
        }

        return $this;
    }

    public function fetchTransactionInfo(Mage_Payment_Model_Info $payment, $transactionId)
    {
        $api        = Mage::getModel('magebinary_binarypay/webpayment');
        $gateway    = $api->getGateway();
        try {
            $transcation = $gateway->getTransaction($transactionId);
        } catch (BinaryPay_Exception $e) {
            Mage::throwException($e->getMessage());
        } catch (Exception $e) {
            Mage::throwException('Something massively went wrong. Please try again. If the problem still exists, please contact us');
            Mage::log($e->getTrace(), null, 'paymark.log', true);
        }
        return $transcation;
    }

    protected function _isValidPurchaseUrl($url)
    {
        if (empty($url)) {
            $message = Mage::helper('magebinary_binarypay')->__('Unable to redirect you to the payment page. Please try again later.');
            Mage::throwException($message);
        }
        return $this;
    }

    protected function _getAmount($isBackend = false)
    {
        if ($isBackend) {
            $amount = Mage::getSingleton('adminhtml/session_quote')->getQuote()->getGrandTotal();
            return $amount;
        }
        $amount    = $this->_getQuote()->getGrandTotal();
        return $amount;
    }
}