<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/
class MageBinary_BinaryPay_Model_System_Config_Directpost_Accountstatus
{
    const AUTHCATION_ERROR = 3000;
    const ACCOUNT_ID_ERROR = 5000;
    const ACTIVATE_FAILED  = 'BINARY_ACTIVIATION_ERROR';
    /**
     * Account details storage
     *
     * @access protected
     * @var bool|array
     */
    protected $_status = false;
    protected $_code   = false;

    /**
     * Set AccountDetails on class property if not already set
     *
     * @return void
     */
    public function __construct()
    {
        try {
            $api         = Mage::getModel('magebinary_binarypay/directpost');
            $directpost  = $api->getGateway();
            if ($api->getPaymentMethodConfig('active')) {
                $testPurchase       =  array(
                    BinaryPay::ACCOUNTID     => '621620',
                    BinaryPay::AMOUNT        => '1022.00',
                    BinaryPay::CARD_NUMBER   => '4111111111111111',
                    BinaryPay::CARD_TYPE     => 'VISA',
                    BinaryPay::CARD_EXPIRY   => '1220',
                    BinaryPay::CARD_HOLDER   => 'John Smith',
                    BinaryPay::CARD_CSC      => '111',
                    BinaryPay::EMAIL         => '',
                    BinaryPay::PARTICULAR    => 'transaction particular',
                    BinaryPay::REFERENCE     => 'transaction reference',
                    BinaryPay::CARD_TOKEN_REF=> 'TokenReference',
                    BinaryPay::STORE_CARD    => 0
                );

                $testPurchase[BinaryPay::ACCOUNTID] = $api->getPaymentMethodConfig(
                    'account_id', true
                );

                $directpost->setConfig(
                    array(
                        BinaryPay::USERNAME => $api->getPaymentMethodConfig(
                            'username', true
                        ),
                        BinaryPay::PASSWORD => $api->getPaymentMethodConfig(
                            'password', true
                        ),
                        BinaryPay::API_KEY  => $api->getPaymentMethodConfig(
                            'apikey', true
                        )
                    )
                );

                $directpost->purchase($testPurchase);
            }

        } catch (BinaryPay_Exception $e) {
            if ($e->getCode() === self::AUTHCATION_ERROR
                || $e->getCode() === self::ACCOUNT_ID_ERROR
                || $e->getCode() === self::ACTIVATE_FAILED) {
                //Error handling here
                $this->_status = $e->getMessage();
                $this->_code   = $e->getCode();
            }
        }
    }

    /**
     * Return data if API key is entered
     *
     * @return array
     */
    public function toOptionArray()
    {
        if ($this->_status || $this->_code) {
            $message = $this->_status;
            $code    = $this->_code;
            $html    = "<h4 style='color:red;font-weight:bold;'>ERROR: $code</h4>
                        <p style='color:red;'>$message</p>";
        } else {
            $html    = "<h4 style='color:green;font-weight:bold;'>Username & Password Validated</h4><pre style='font-size:10px; color:grey;'>Make sure your account Id valid by placing an test order.</pre>";
        }
        return array(
                'label' => Mage::helper('magebinary_binarypay')->__($html)
            );
    }

}
