<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/
class MageBinary_BinaryPay_Model_System_Config_MerchantUrlStatus
{
    /**
     * Merchant Url Status
     *
     * @access protected
     * @var bool
     */
    protected $_status  = false;

    /**
     * Set AccountDetails on class property if not already set
     *
     * @return void
     */
    public function __construct()
    {
        if (!$this->_status) {
            try {
                $api            = Mage::getModel('magebinary_binarypay/onlineeftpos');
                $gateway        = $api->getGateway();
                $url            = Mage::getBaseUrl();
                $gateway->isUrlValid($url);
            } catch (BinaryPay_Exception $e) {
                Mage::log($e->getMessage());
                if ($e->getCode() === BinaryPay::ERROR_SIGNAL) {
                    //Error handling here
                    $this->_status = $e->getMessage();
                }
            }
        }
    }

    /**
     * Return data if API key is entered
     *
     * @return array
     */
    public function toOptionArray()
    {
        if ($this->_status) {
            $message = $this->_status;
            $html    = "<h4 style='color:red;font-weight:bold;'>ERROR: $message</h4>";
        } else {
            $html    = "<h4 style='color:green;font-weight:bold;'>Url Validated</h4><pre style='font-size:10px; color:grey;'>Ensure Paymark API can send messages to your server.</pre>";
        }
        return Mage::helper('magebinary_binarypay')->__($html);
    }

}
