<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/
class MageBinary_BinaryPay_Model_System_Config_Onlineeftpos_Accountstatus
{
    const CLIENT_ERROR      = 'invalid_client';
    const MERCHANT_ID       = 'merchantIdCode';
    const SERVER_ERROR      = 'server error';
    const AUTHENTICATION    = 'authentication';
    /**
     * Account details storage
     *
     * @access protected
     * @var bool|array
     */
    protected $_status = false;
    protected $_code   = false;

    /**
     * Set AccountDetails on class property if not already set
     *
     * @return void
     */
    public function __construct()
    {
        try {
            $api            = Mage::getModel('magebinary_binarypay/onlineeftpos');
            $gateway        = $api->getGateway();

            if ($api->getPaymentMethodConfig('active')) {
                $array = array(
                    BinaryPay::MOBILENUMBER  => '0210123456',
                    BinaryPay::PAYMENT_TYPE  => "MOBILE",
                    BinaryPay::BANK_ID       => "ASB",
                    BinaryPay::ACCOUNTID     => "300000005",
                    BinaryPay::RETURN_URL    => "http://google.com",
                    BinaryPay::MERCHANT_URL  => "http://dev.magebinary.com",
                    BinaryPay::AMOUNT        => 10000,
                    BinaryPay::CURRENCY      => "NZD",
                    BinaryPay::DESCRIPTION   => "",
                    BinaryPay::REFERENCE     => "199",
                    BinaryPay::USER_AGENT    => "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.106 Safari/537.36",
                    BinaryPay::IP            => "192.168.0.1"
                );

                $array[BinaryPay::ACCOUNTID] = $api->getPaymentMethodConfig('paymark_account_id', true);

                $gateway->setConfig(
                    array(
                        BinaryPay::USERNAME => $api->getPaymentMethodConfig('username', true),
                        BinaryPay::PASSWORD => $api->getPaymentMethodConfig('password', true),
                    )
                );
                $gateway->purchase($array);
            }
        } catch (BinaryPay_Exception $e) {
            $this->_handleException($e);
        }
    }

    protected function _handleException($e)
    {
        $error      = false;
        $code       = $e->getCode();
        $message    = $e->getMessage();

        if ($code == self::AUTHENTICATION) {
            $error = true;
        }

        // If user entered incorrect merchant id
        if (strpos($message, self::MERCHANT_ID)) {
            $error = true;
        }

        // If user entered incorrect username & password
        if ($code == self::CLIENT_ERROR) {
            $error = true;
        }

        // If user entered sandbox account into production environment
        if ($code == self::SERVER_ERROR) {
            $error = true;
        }

        if ($error) {
            $this->_status  = $e->getMessage();
            $this->_code    = $e->getCode();
        }
    }

    /**
     * Return data if API key is entered
     *
     * @return array
     */
    public function toOptionArray()
    {
        if ($this->_status && $this->_code) {
            $message = $this->_status;
            $code    = $this->_code;
            $html    = "<h4 style='color:red;font-weight:bold;'>ERROR: $code</h4>
                        <p style='color:red;'>$message</p>";
        } else {
            $html    = "<h4 style='color:green;font-weight:bold;'>Username & Password Validated</h4><pre style='font-size:10px; color:grey;'>Make sure your account Id valid by placing an test order.</pre>";
        }
        return array(
                'label' => Mage::helper('magebinary_binarypay')->__($html)
            );
    }

}
