<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/

class MageBinary_BinaryPay_Adminhtml_CustomerController extends MageBinary_BinaryPay_Controller_Adminhtml_Abstract
{
    protected $_code = 'directpost';

    /**
     * Delete credit card
     */
    public function deleteCardAction()
    {
        // If key is invalid, redirect to dashboard
        if (!$this->_validateFormKey()) {
            return $this->_redirect('*/*/');
        }

        $api            = $this->_getApi();
        $tokenReference = $this->getRequest()->getParam('token_reference');

        // Validate user request and login status
        if (empty($tokenReference) && Mage::getSingleton('admin/session')->isLoggedIn()) {
            $this->_redirect('/');
            return;
        }

        try {
            // Register current customer to the registry
            $this->_initCustomer();

            $api->deleteTokenByReference($tokenReference);

            $result = array(
                'status'    => 'success',
                'message'   => 'The credit card is successfully deleted.'
            );
        } catch (BinaryPay_Exception $e) {
            $result = array('status' => 'error', 'message' => $e->getMessage());
        } catch (Exception $e) {
            $result = array('status' => 'error', 'message' => $e->getMessage());
            Mage::log($e->getMessage(), Zend_Log::ERR);
        }

        return $this->_sendMessage($result);
    }

    public function addCardAction()
    {
        $api        = $this->_getApi();
        $request    = $this->getRequest();


        //Check if it is the right user
        if (!$this->_validateFormKey()) {
            $this->_redirect('*/*/');
            return;
        }

        $ccNumber = $request->getPost('cc_number');

        $info = array(
            BinaryPay::CARD_NUMBER => $ccNumber,
            BinaryPay::CARD_EXPIRY => $api->getCardExpiryByMthAndYear(
                $request->getPost('cc_exp_month'),
                $request->getPost('cc_exp_year')
            )
        );

        // Build card information
        $cardInfo = array(
            BinaryPay::CARD_NUMBER     => $ccNumber,
            BinaryPay::CARD_TYPE       => $api->translateToPaymarkCcType($request->getPost('cc_type')),
            BinaryPay::CARD_HOLDER     => $request->getPost('cc_owner'),
            BinaryPay::CARD_EXPIRY     => $api->getCardExpiryByMthAndYear(
                                                    $request->getPost('cc_exp_month'),
                                                    $request->getPost('cc_exp_year')
                                                  ),
            BinaryPay::CARD_TOKEN_REF  => $tokenReference = $api->generateCreditCardTokenReference($info)
        );

        foreach ($cardInfo as $key => $value) {
            if (empty($value)) {
                $result = array(
                    'status'    => 'error',
                    'message'   => "Invalid key $key, value can not be empty"
                );
                return $this->_sendMessage($result);
            }
        }

        try {
            $this->_initCustomer();
            $customer      = Mage::registry('current_customer');
            $response      = $api->getGateway()->add($cardInfo);
            $api->saveToken($response);

            //Passing the delete url to frontend for create delete button
            $deleteCardUrl = $this->getUrl('*/*/deletecard/token_reference/' . $tokenReference . "/id/" . $customer->getId());

            //Passing the logo url to frontend for create logo
            $cardImgUrl    = Mage::getModel('magebinary_binarypay/system_general')->getCardLogoFileNameByCardType($api->translateToPaymarkCcType($response['cardType']));
            //return an array for ajax to process
            $result = array(
                'status'        => 'success',
                'message'       => 'The credict card is successfully saved.',
                'cardNumber'    => $response['cardNumber'],
                'cardType'      => $api->getCardNameByCardType($response['cardType']),
                'cardExpiry'    => $response['cardExpiry'],
                'cardHolder'    => $response['cardHolder'],
                'deleteCardUrl' => $deleteCardUrl,
                'cardImgUrl'    => $cardImgUrl
            );

        } catch (BinaryPay_Exception $e) {
            $result = array('status' => 'error', 'message' => $e->getMessage());
        } catch (Exception $e) {
            $result = array('status' => 'error', 'message' => $e->getMessage());
            Mage::log($e->getMessage());
        }
        return $this->_sendMessage($result);
    }
}