<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/

class MageBinary_BinaryPay_Adminhtml_MobilesController extends MageBinary_BinaryPay_Controller_Adminhtml_Abstract
{
    protected $_code = 'onlineeftpos';

    public function addTkAction()
    {
        // Check if it is the right user
        if (!$this->_validateFormKey()) {
            return $this->_redirect('*/*/');
        }

        try {
            $request    = $this->getRequest();
            $postData   = $request->getPost();

            $mobileNumber   = $this->getRequest()->getParam('mobile_number');

            $api = $this ->_getApi();

            // Generate token reference number by mobile number
            $tokenReference = $api->generateTokenReference($mobileNumber);

            // Set token reference number into the post data array
            $postData['token_reference'] = $tokenReference;

            $this->_initCustomer();

            $customer = Mage::registry('current_customer');

            $api->saveToken($postData);

            // Generate the delete Url at the same time
            $deleteUrl = $this->getUrl(
                'adminhtml/mobiles/deleteTk',
                array(
                    'token_reference'   => $tokenReference,
                    'id'                => $customer->getId()
                )
            );

            // Return an array for ajax to process
            $result = array(
                'status'        => 'success',
                'message'       => 'The mobile number is successfully saved.',
                'mobileNumber'  => $mobileNumber,
                'deleteUrl'     => $deleteUrl,
            );
        } catch (BinaryPay_Exception $e) {
            $result = array('status' => 'error', 'message' => $e->getMessage());
        } catch (Exception $e) {
            $result = array('status' => 'error', 'message' => $e->getMessage());
            Mage::log($e->getMessage(), Zend_Log::ERR);
        }

        $this->_sendMessage($result);
    }

    /**
     * Delete tokenreference
     */
    public function deleteTkAction()
    {
        // If form key is invalid, redirect to dashboard
        if (!$this->_validateFormKey()) {
            return $this->_redirect('*/*/');
        }

        try {
            $api            = $this->_getApi();
            $tokenReference = $this->_getTokenReference();

            // Validate user request and login status
            if (empty($tokenReference) && Mage::getSingleton('admin/session')->isLoggedIn()) {
                return $this->_redirect('/');
            }

            // Register current customer to the registry
            $this->_initCustomer();

            $api->deleteTokenByReference($tokenReference);

            $result = array(
                'status'    => 'success',
                'message'   => 'The mobile is successfully deleted.'
            );

        } catch (BinaryPay_Exception $e) {
            $result = array('status' => 'error', 'message' => $e->getMessage());
        } catch (Exception $e) {
            $result = array('status' => 'error', 'message' => $e->getMessage());
            Mage::log($e->getMessage(), Zend_Log::ERR);
        }

        $this->_sendMessage($result);
    }
}