<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/
include_once 'Mage/Adminhtml/controllers/Sales/Order/CreateController.php';

class MageBinary_BinaryPay_Adminhtml_Sales_Order_CreateController extends Mage_Adminhtml_Sales_Order_CreateController
{
    public function saveAction()
    {
        $request    = $this->getRequest();
        $payment    = $request->getParam('payment');
        $method     = isset($payment['method']) ? $payment['method'] : '';
        $cardToken  = isset($payment['card_token']) ? $payment['card_token'] : '';

        // Does not need card token for Q card
        if ($method === MageBinary_BinaryPay_Model_Config::QCARD_FLEXILONG) {
            $cardToken = '';
        }

        try {
            $this->_validateRequest();
            if (($method === MageBinary_BinaryPay_Model_Config::PAYMARK_WEBPAYMENT && empty($cardToken))
                || $method === MageBinary_BinaryPay_Model_Config::QCARD_FLEXILONG) {

                $methodName = explode('_', $method);
                $modelName  = next($methodName);

                $order = $this->_getOrderCreateModel()
                              ->setIsValidate(true)
                              ->importPostData($this->getRequest()->getPost('order'));

                Mage::getModel('core/cookie')->set('quoteEntityId', $order->getQuote()->getEntityId());

                if ($method === MageBinary_BinaryPay_Model_Config::QCARD_FLEXILONG) {
                    Mage::getModel('core/cookie')->set('flexiTerm', $payment['flexi_term']);
                }

                $this->_getSession()->clear();

                $purchaseUrl = Mage::getModel("magebinary_binarypay/method_$modelName")->getPurchaseUrl(true);

                // Redirect to Purchase Url
                return $this->_redirectUrl($purchaseUrl);
            }
        } catch (BinaryPay_Exception $e) {
            $message = $e->getMessage();
            if (!empty($message)) {
                $this->_getSession()->addError($message);
            }
            $this->_redirect('*/*/');
        } catch (Exception $e) {
            $this->_getSession()->addException($e, $this->__('Order saving error: %s', $e->getMessage()));
            $this->_redirect('*/*/');
        }

        // user is not using paymark webpayment external API
        parent::saveAction();
    }

    protected function _validateRequest()
    {
        $request    = $this->getRequest();
        $payment    = $request->getParam('payment');
        $method     = isset($payment['method']) ? $payment['method'] : '';

        if (!isset($payment) || empty($method)) {
            Mage::throwException('Undefined Payment Method');
        }

        return $this;
    }
}
