<?php

/**
 * Magento BinaryPay Payment Extension
 *
 * NOTICE OF LICENSE
 *
 * Copyright 2017 - 2019 MageBinary
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @category    MageBinary
 * @package     MageBinary_BinaryPay
 * @author      MageBinary Team
 * @copyright   Copyright (c) 2017 - 2019 MageBinary (http://www.magebinary.com)
 * @license     http://www.apache.org/licenses/LICENSE-2.0
 */
class MageBinary_BinaryPay_LatitudepayController extends MageBinary_BinaryPay_Controller_Abstract
{

    /**
     * @var bool
     */
    protected $_debug = true;

    /**
     * $_successStatuses
     * @var array
     */
    protected $_successStatuses = [
        MageBinary_BinaryPay_Model_Method_Latitudepay::SUCCESSFUL
    ];

    /**
     * $_pendingStatuses
     * @var array
     */
    protected $_pendingStatuses = [
        MageBinary_BinaryPay_Model_Method_Latitudepay::UNKNOWN
    ];

    public function returnAction()
    {
        $session = $this->_getCheckoutSession();

        try {
            $this->_apiResponse = $response = $this->getRequest()->getQuery();
            // Pass Response To Capture
            $session->setResponse($response);

            if ($this->_debug) {
                Mage::log('RESPONSE FROM LATITUDEPAY', Zend_Log::DEBUG, 'BinaryPay.log', true);
                Mage::log(json_encode($response), Zend_Log::DEBUG, 'BinaryPay.log', true);
                Mage::log('END OF RESPONSE', Zend_Log::DEBUG, 'BinaryPay.log', true);
            }

            $this->_validateResponse();

            if ($this->isBackendOrder()) {
                // $this->createOrderForBackend($quote);
            }

            $this->placeOrder();

        } catch (BinaryPay_Exception $e) {
            $this->addErrorException($session, $e);
            $this->_redirectUrl = MageBinary_BinaryPay_Model_Config::CHECKOUT_CART_PAGE_URL;
            Mage::logException($e);
        } catch (Mage_Core_Exception $e) {
            $this->addErrorException($session, $e);
            $this->_redirectUrl = MageBinary_BinaryPay_Model_Config::CHECKOUT_CART_PAGE_URL;
            Mage::logException($e);
        } catch (Exception $e) {
            $this->addErrorException($session, $e);
            $this->_redirectUrl = MageBinary_BinaryPay_Model_Config::CHECKOUT_CART_PAGE_URL;
            Mage::logException($e);
        }

        $session->unsIsPending();
        $session->unsResponse();
        $session->unsPaymentStatus();

        return $this->_redirect($this->_redirectUrl);
    }

    /**
     * Validate Response from Paymark webpayment
     * @param Array $request - Response From API
     * @return Boolean
     */
    protected function _validateResponse()
    {
        $session = $this->_getCheckoutSession();
        $response = $this->_apiResponse;

        // Check return response
        // Check request types & origin
        if (!$this->getRequest()->isGet()) {
            $session->setIsPending(true);
            Mage::throwException($this->__('Warning: The response is not a GET request'));
        }

        $callbackData = $this->getRequest()->getParam('callback_data');
        Mage::log('CALLBACK DATA FROM LATITUDEPAY', Zend_Log::DEBUG, 'BinaryPay.log', true);
        Mage::log($callbackData, Zend_Log::DEBUG, 'BinaryPay.log', true);
        Mage::log('END OF CALLBACK', Zend_Log::DEBUG, 'BinaryPay.log', true);

        Mage::helper('magebinary_binarypay/payment')->validateCallbackData($callbackData);
        Mage::helper('magebinary_binarypay/payment')->validateSignature($response,'latitudepay');
        Mage::helper('magebinary_binarypay/payment')->validateTotalAmount();

        $status = isset($this->_apiResponse['result']) ? $this->_apiResponse['result'] : null;
        $session->setPaymentStatus($status);

        // Not paid or pending payment status then redirect customer to cart page
        if (!$this->isPaid($status) && !$this->isPending($status)) {
            throw new BinaryPay_Exception($this->translateErrorMessages($response));
        }

        if ($this->isPending($status)) {
            Mage::throwException('The payment response has been received but with an UNKNOWN status.');
        }

        return $this;
    }

    public function translateErrorMessages($response)
    {
        $status = $response['result'];
        switch ($status) {
            case MageBinary_BinaryPay_Model_Method_Latitudepay::FAILED:
                $message = $this->__($response['message']);
                break;
            default:
                $message = $this->__($response['message']);
                break;
        }
        return $message;
    }
}