<?php
/**
* Magento BinaryPay Payment Extension
*
* NOTICE OF LICENSE
*
* Copyright 2017 MageBinary
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
* @category    MageBinary
* @package     MageBinary_BinaryPay
* @author      MageBinary Team
* @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
* @license     http://www.apache.org/licenses/LICENSE-2.0
*/
class MageBinary_BinaryPay_MobilesController extends MageBinary_BinaryPay_Controller_Abstract
{
    protected $_gateway = 'onlineeftpos';

    public function indexAction()
    {
        $this->loadLayout();
        $this->_initLayoutMessages('catalog/session');

        if ($navigationBlock = $this->getLayout()->getBlock('customer_account_navigation')) {
            $navigationBlock->setActive('binarypay/mobiles');
        }
        $this->getLayout()->getBlock('head')->setTitle($this->__('My Mobiles'));

        $this->renderLayout();
    }

    public function addAction()
    {
        // Successful added card
        $result = array(
            'status'    => 'success',
            'message'   => 'The mobile is successfully saved.'
        );

        // Validate request data
        if (!$this->_validateAddRequest()) {
            return;
        }

        try {
            // Get getGateway dynamic
            $this->_prepareGetGateway();

            // Build card information
            $data = $this->_buildDataInfo();

            // Validate card info after build
            $this->_validateDataInfo();

            $api = $this->_getApi();

            // Save token will convert response to Varien Object
            $api->saveToken($data);

        } catch (BinaryPay_Exception $e) {
            $result = array(
                'status' => 'error',
                'message' => $e->getMessage()
            );
        } catch (Exception $e) {
            $result = array(
                'status' => 'error',
                'message' => $e->getMessage()
            );
            Mage::log($e->getMessage(), Zend_Log::ERR);
        }

        $this->_redirectAfterSuccess()->_sendMessage($result);
        return $this;
    }

    public function deleteAction()
    {
        $result = array();

        if (!$this->_validateFormKey()) {
            return $this->_redirect('*/*/');
        }

        try {
            // validate requested data is empty
            $this->_prepareGetGateway();

            // Validate delete request
            $this->_validateDeleteRequest();

            $api = $this->_getApi();

            $tokens = $api->getCustomerMobiles();

            $api->deleteTokenByReference($this->_tokenRef);
            $result = array('status' => 'success', 'message' => 'The mobile is successfully deleted.');
        } catch (BinaryPay_Exception $e) {
            $result = array('status' => 'error', 'message' => $e->getMessage());
        } catch (Exception $e) {
            $result = array('status' => 'error', 'message' => $e->getMessage());
            Mage::log($e->getMessage());
        }

        // redirect page after success
        $this->_redirectAfterSuccess()->_sendMessage($result);

        return $this;
    }

    protected function _validateAddRequest()
    {
        // If the request is not post, just render the form
        $request    = $this->getRequest();
        if ($request->isGet()) {
            $this->loadLayout();
            $this->_initLayoutMessages('catalog/session');
            if ($navigationBlock = $this->getLayout()->getBlock('customer_account_navigation')) {
                $navigationBlock->setActive('binarypay/mobiles/add');
            }
            $this->getLayout()->getBlock('head')->setTitle($this->__('Add Your Mobile Number'));
            $this->renderLayout();
            return false;
        } else {
            // Validate form key
            if (!$this->_validateFormKey()) {
                $result = array('status' => 'error', 'message' => "Invalid User");
                $this->_sendMessage($result);
            }
        }

        return $this;
    }

    protected function _buildDataInfo()
    {
        $request    = $this->getRequest();
        $api        = $this->_getApi();
        $data       = $request->getPost();

        if (isset($data['form_key'])) {
            unset($data['form_key']);
        }

        if (isset($data['mobile_number'])) {
            $tokenRef = $api->generateTokenReference($data['mobile_number']);
            $data['token_reference'] = $tokenRef;
        }

        return $this->_data = $data;
    }
}