<?php
/**
 * Magento BinaryPay Payment Extension
 *
 * NOTICE OF LICENSE
 *
 * Copyright 2017 MageBinary
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @category    MageBinary
 * @package     MageBinary_BinaryPay
 * @author      MageBinary Team
 * @copyright   Copyright (c) 2017 MageBinary (http://www.magebinary.com)
 * @license     http://www.apache.org/licenses/LICENSE-2.0
 */
class MageBinary_BinaryPay_TokensController extends MageBinary_BinaryPay_Controller_Abstract
{
    /**
     * $_gateway - gateway name for tokensController
     * @var string
     */
    protected $_gateway = 'directpost';

    /**
     * @var Magebinary_Paymark_Model_Checkout
     */
    protected $_checkout;

    /**
     * @var Mage_Sales_Model_Quote
     */
    protected $_quote = false;

    /**
     * @var array
     */
    protected $_apiResponse;

    /**
     * $_successStatuses
     * @var array
     */
    protected $_successStatuses = [
        MageBinary_BinaryPay_Model_Method_Directpost::SUCCESSFUL,
        MageBinary_BinaryPay_Model_Method_Directpost::INPROGRESS
    ];

    /**
     * $_pendingStatuses
     * @var array
     */
    protected $_pendingStatuses = [
        MageBinary_BinaryPay_Model_Method_Directpost::UNKNOWN
    ];

    /**
     * indexAction - render page content for paymark/tokens/index
     */
    public function indexAction()
    {
        $this->loadLayout();
        $this->_initLayoutMessages('catalog/session');

        if ($navigationBlock = $this->getLayout()->getBlock('customer_account_navigation')) {
            $navigationBlock->setActive('binarypay/tokens');
        }
        $this->getLayout()->getBlock('head')->setTitle($this->__('My Credit Cards'));

        $this->renderLayout();
    }

    /**
     * returnAction
     * Handle the response from Paymark
     */
    public function returnAction()
    {
        $request   = $this->getRequest();
        $session    = $this->_getCheckoutSession();

        $particular = $session->getParticular();
        if (!isset($particular)) {
            /**
            if ($customerId = $request->getQuery('CID')) {
                Mage::getSingleton('customer/session')->renewSession()->loginById($customerId);
            }

             if ($quoteId = $request->getQuery('QID')) {
               $session->setQuoteId($quoteId);
            }
            **/
            //$session->clear();
            $sessionId = $request->getQuery('SID');
            $sessionCId = $request->getQuery('SCID');
            $cookie = Mage::getSingleton('core/cookie');
            $time = 86400;
            $cookie->set('frontend', '');
            $cookie->set('frontend_cid', '');
            $cookie->set('frontend', $sessionId , $time);
            $cookie->set('frontend_cid', $sessionCId , $time);
            $session    = $this->_getCheckoutSession();
            Mage::log('session renewed');
            Mage::log($sessionId);
            Mage::log($session->getParticular());
           //Mage::log(Mage::getSingleton('checkout/session')->getParticular());
            die('Refreshing...Please do not close this window. <script>setTimeout(function() {location.reload(true);}, 2000);</script>');
        }

        try {
            $api            = Mage::getModel('magebinary_binarypay/webpayment');
            $gateway        = $api->getGateway();
            $response       = $gateway->camelize($request->getPost());

            // Pass Response To Capture
            $session->setResponse($response);

            if ($this->_debug) {
                Mage::log('RESPONSE FROM Paymark', Zend_Log::DEBUG, 'BinaryPay.log', true);
                Mage::log(json_encode($response), Zend_Log::DEBUG, 'BinaryPay.log', true);
                Mage::log('END OF RESPONSE', Zend_Log::DEBUG, 'BinaryPay.log', true);
            }

            $this->_validateResponse($request);

            // User wants to save card
            if (isset($response['cardToken']) && $response['cardToken']) {
                $cardInfo = array(
                    BinaryPay::CARD_TOKEN       => $response['cardToken'],
                    BinaryPay::CARD_NUMBER      => $response['cardNumber'],
                    BinaryPay::CARD_TYPE        => $response['cardType'],
                    BinaryPay::CARD_EXPIRY      => $response['cardExpiry'],
                    BinaryPay::CARD_HOLDER      => $response['cardHolder'],
                    BinaryPay::CARD_TOKEN_REF   => $api->generateCreditCardTokenReference($response),
                );
                $api->saveToken($cardInfo);
            }

            if ($this->isBackendOrder()) {
                $this->createOrderForBackend($quote);
            }

            $this->placeOrder();

        } catch (BinaryPay_Exception $e) {
            $this->addErrorException($session, $e);
            $this->_redirectUrl = MageBinary_BinaryPay_Model_Config::CHECKOUT_CART_PAGE_URL;
            Mage::logException($e);
        } catch (Mage_Core_Exception $e) {
            $this->addErrorException($session, $e);
            $this->placeOrder($e);
            Mage::logException($e);
        } catch (Exception $e) {
            $this->addErrorException($session, $e);
            $this->placeOrder($e);
            Mage::logException($e);
        }
        $session->unsIsPending();
        $session->unsResponse();
        $session->unsPaymentStatus();

        return $this->_redirect($this->_redirectUrl);
    }

    /**
     * Validate Response from Paymark webpayment
     * @param  Array $request - Response From API
     * @return Boolean
     */
    protected function _validateResponse($request)
    {
        //Be really really careful with how to get checkout session. Have to be this to get the user's session.
        $session    = $this->_getCheckoutSession();
        $cookie     = Mage::getModel('core/cookie');
        $particular = $session->getParticular();
        $response   = Mage::helper('magebinary_binarypay')->camelize($request->getPost());
        $quote      = $this->_getQuote();

       if (!isset($particular)) {
           //TODO: Magento drops session sometimes as said 
            //https://magento.stackexchange.com/questions/215292/security-patch-supee-10570-possible-issues
           //Mage::log($session);
       }

        // Prevent user reuse the session
        $session->unsetData('particular');

        // Check return response
        // Check request types & origin
        if (!$request->isPost()) {
            $session->setIsPending(true);
            Mage::throwException($this->__('Warning: The response is not a POST request'));
        }

        /* This step is avoided when admin is placing the order*/
        // Check with API & Transcation particular
        if (!isset($particular) || $particular != $response['particular']) {
            // if the order is coming from the backend
            if ($this->isBackendOrder()) {
                $particular = $cookie->get('particular');
                // remove particular value in the cookie, to avoid abuse
                $cookie->delete('particular');
                // Check if user comming from backend
                if ($particular !== null && $response['particular'] != $particular) {
                    $message = 'Something went wrong. Paymark can not verify your order/payment. Please contact us.';
                    Mage::throwException($this->__($message));
                }
            } else {
                $message = 'Something went wrong. Paymark can not verify your order/payment. Please contact us.';
                Mage::throwException($this->__($message));
            }
        }

        // Prevent user update cart after redirecting
        // Check Amount and Quote Amount
        if (!isset($response['amount']) || $response['amount'] != $quote->getGrandTotal()) {
            Mage::throwException($this->__('The order and payment amount cannot be matched up correctly.'));
        }

        $status = $response['status'];
        $session->setPaymentStatus($status);

        // Not paid or pending payment status then redirect customer to cart page
        if (!$this->isPaid($status) && !$this->isPending($status)) {
            throw new BinaryPay_Exception($this->translateErrorMessages($response));
        }

        if ($this->isPending($status)) {
            Mage::throwException('The payment response has been received but with UNKNOWN status.');
        }

        return $this;
    }

    public function translateErrorMessages($response)
    {
        $status = $response['status'];
        switch ($status) {
            case MageBinary_BinaryPay_Model_Method_Directpost::CANCELLED:
                $message = $this->__('You have canceled the payment at checkout. Please contact us for more assistance or select another payment method to complete checkout.');
                break;
            default:
                $message = $this->__($response['errorMessage']);
                break;
        }
        return $message;
    }


    protected function _validateAddRequest()
    {
        // If the request is not post, just render the form
        $request = $this->getRequest();
        if ($request->isGet()) {
            $this->loadLayout();
            $this->_initLayoutMessages('catalog/session');
            if ($navigationBlock = $this->getLayout()->getBlock('customer_account_navigation')) {
                $navigationBlock->setActive('binarypay/tokens/add');
            }
            $this->getLayout()->getBlock('head')->setTitle($this->__('Add Your Credit Card'));
            $this->renderLayout();
            return false;
        } else {
            // Validate form key
            if (!$this->_validateFormKey()) {
                $result = array('status' => 'error', 'message' => "Invalid User");
                $this->_sendMessage($result);
            }
        }
        return $this;
    }

    protected function _buildCardInfo()
    {
        $request    = $this->getRequest();
        $api        = $this->_getApi();

        $ccNumber   = $request->getPost('cc_number');
        $cardExpiry = $api->getCardExpiryByMthAndYear(
            $request->getPost('cc_exp_month'),
            $request->getPost('cc_exp_year')
        );
        $cardType = $api->translateToPaymarkCcType($request->getPost('cc_type'));
        $cardOwner = $request->getPost('cc_owner');

        $info = array(
            BinaryPay::CARD_NUMBER => $ccNumber,
            BinaryPay::CARD_EXPIRY => $cardExpiry,
        );

        $cardTokenRef = $api->generateCreditCardTokenReference($info);

        $cardInfo = array(
            BinaryPay::CARD_NUMBER  => $ccNumber,
            BinaryPay::CARD_TYPE    => $cardType,
            BinaryPay::CARD_HOLDER  => $cardOwner,
            BinaryPay::CARD_EXPIRY  => $cardExpiry,
            BinaryPay::CARD_TOKEN_REF => $cardTokenRef,
        );

        $this->_data = $cardInfo;

        return $this->_data;
    }

    public function deleteAction()
    {
        if (!$this->_validateFormKey()) {
            return $this->_redirect('*/*/');
        }

        // validate requested data is empty
        $this->_prepareGetGateway();

        // Validate delete request
        $this->_validateDeleteRequest();

        $api = $this->_getApi();

        // If directpost not actived, use webpayment's credientials
        if (!$api->getPaymentMethodConfig('active') ) {
            $this->_setGateway('webpayment');
            // Get new gateway's object
            $api = $this->getApi();
        }

        try {
            $api->deleteTokenByReference($this->_tokenRef);
            $result = array('status' => 'success', 'message' => 'The credit card is successfully deleted.');
        } catch (BinaryPay_Exception $e) {
            $result = array('status' => 'error', 'message' => $e->getMessage());
        } catch (Exception $e) {
            $result = array('status' => 'error', 'message' => $e->getMessage());
            Mage::log($e->getMessage());
        }

        // redirect page after success
        $this->_redirectAfterSuccess()->_sendMessage($result);

        return $this;
    }

    public function addAction()
    {
        // Successful added card
        $result = array(
            'status' => 'success',
            'message' => 'The credit card is successfully saved.',
        );

        // Validate request data
        if (!$this->_validateAddRequest()) {
            return;
        }

        try {
            // Get getGateway dynamic
            $this->_prepareGetGateway();

            // Build card information
            $data = $this->_buildCardInfo();

            // Validate card info after build
            $this->_validateDataInfo();

            $api = $this->_getApi();

            // If directpost not actived, use webpayment's credientials
            if (!$api->getPaymentMethodConfig('active') ) {
                $this->_setGateway('webpayment');
                // Get new gateway's object
                $api = $this->getApi();
            }

            // Add card to magento and api
            $response = $api->getGateway()->add($data);

            // Save token will convert response to Varien Object
            $api->saveToken($response);

        } catch (BinaryPay_Exception $e) {
            $result = array(
                'status' => 'error',
                'message' => $e->getMessage(),
            );
        } catch (Exception $e) {
            $result = array(
                'status' => 'error',
                'message' => $e->getMessage(),
            );
            Mage::log($e->getMessage(), Zend_Log::ERR);
        }

        $this->_redirectAfterSuccess()->_sendMessage($result);
        return $this;
    }
}